@@ -1,5 +1,5 @@
 #! /bin/sh
-# $Header: /cvsroot/pgsql-server/src/test/regress/pg_regress.sh,v 1.36 2003/11/02 21:56:15 tgl Exp $
+# $PostgreSQL: pgsql/src/test/regress/pg_regress.sh,v 1.53.4.1 2005/07/17 18:29:13 tgl Exp $
 
 me=`basename $0`
 : ${TMPDIR=/tmp}
@@ -24,6 +24,7 @@ Options:
 
 Options for \`temp-install' mode:
   --top-builddir=DIR        (relative) path to top level build directory
+  --temp-port=PORT          port number to start temp postmaster on
 
 Options for using an existing installation:
   --host=HOST               use postmaster running on HOST
@@ -50,6 +51,14 @@ message(){
 
 unset LC_COLLATE LC_CTYPE LC_MONETARY LC_MESSAGES LC_NUMERIC LC_TIME LC_ALL LANG LANGUAGE
 
+# On Windows the default locale may not be English, so force it
+case $host_platform in
+    *-*-cygwin*|*-*-mingw32*)
+	LANG=en
+	export LANG
+	;;
+esac
+
 
 # ----------
 # Check for echo -n vs echo \c
@@ -95,6 +104,7 @@ unset multibyte
 dbname=regression
 hostname=localhost
 maxconnections=0
+temp_port=65432
 
 : ${GMAKE='@GMAKE@'}
 
@@ -140,6 +150,9 @@ do
         --top-builddir=*)
                 top_builddir=`expr "x$1" : "x--top-builddir=\(.*\)"`
                 shift;;
+        --temp-port=*)
+                temp_port=`expr "x$1" : "x--temp-port=\(.*\)"`
+                shift;;
         --host=*)
                 PGHOST=`expr "x$1" : "x--host=\(.*\)"`
                 export PGHOST
@@ -184,11 +197,10 @@ esac
 
 
 # ----------
-# When on QNX or BeOS, don't use Unix sockets.
+# On some platforms we can't use Unix sockets.
 # ----------
-
 case $host_platform in
-    *-*-qnx* | *beos*)
+    *-*-cygwin* | *-*-mingw32* | *-*-qnx* | *beos*)
         unix_sockets=no;;
     *)
         unix_sockets=yes;;
@@ -230,18 +242,20 @@ PGDATESTYLE='ISO, MDY'; export PGDATESTYLE
 # with the result of the last shell command before the `exit'.  Hence
 # we have to write `(exit x); exit' below this point.
 
-trap '
-    savestatus=$?
+exit_trap(){ 
+    savestatus=$1
     if [ -n "$postmaster_pid" ]; then
         kill -2 "$postmaster_pid"
         wait "$postmaster_pid"
         unset postmaster_pid
     fi
     rm -f "$TMPFILE" && exit $savestatus
-' 0
+}
 
-trap '
-    savestatus=$?
+trap 'exit_trap $?' 0
+
+sig_trap() {
+    savestatus=$1
     echo; echo "caught signal"
     if [ -n "$postmaster_pid" ]; then
         echo "signalling fast shutdown to postmaster with pid $postmaster_pid"
@@ -250,7 +264,9 @@ trap '
         unset postmaster_pid
     fi
     (exit $savestatus); exit
-' 1 2 13 15
+}
+
+trap 'sig_trap $?' 1 2 13 15
 
 
 
@@ -294,12 +310,20 @@ LOGDIR=$outputdir/log
 if [ x"$temp_install" != x"" ]
 then
     if echo x"$temp_install" | grep -v '^x/' >/dev/null 2>&1; then
-        temp_install="`pwd`/$temp_install"
+        case $host_platform in
+          *-*-mingw32*)
+                pkglibdir="`pwd -W`/$temp_install/install/$pkglibdir"
+                temp_install="`pwd`/$temp_install"
+                ;;
+          *)
+                temp_install="`pwd`/$temp_install"
+                pkglibdir=$temp_install/install/$pkglibdir
+                ;;
+        esac
     fi
 
     bindir=$temp_install/install/$bindir
     libdir=$temp_install/install/$libdir
-    pkglibdir=$temp_install/install/$pkglibdir
     datadir=$temp_install/install/$datadir
     PGDATA=$temp_install/data
 
@@ -311,7 +335,13 @@ then
         unset PGHOST
         unset PGHOSTADDR
     fi
-    PGPORT=65432
+
+    # since Makefile isn't very bright, check for out-of-range temp_port
+    if [ "$temp_port" -ge 1024 -a "$temp_port" -le 65535 ] ; then
+	PGPORT=$temp_port
+    else
+	PGPORT=65432
+    fi
     export PGPORT
 
     # Get rid of environment stuff that might cause psql to misbehave
@@ -320,8 +350,9 @@ then
 
     # ----------
     # Set up shared library paths, needed by psql and pg_encoding
-    # (if you run multibyte).  LD_LIBRARY_PATH covers many platforms,
-    # feel free to account for others as well.
+    # (if you run multibyte).  LD_LIBRARY_PATH covers many platforms.
+    # DYLD_LIBRARY_PATH works on Darwin, and maybe other Mach-based systems.
+    # Feel free to account for others as well.
     # ----------
 
     if [ -n "$LD_LIBRARY_PATH" ]; then
@@ -331,12 +362,19 @@ then
     fi
     export LD_LIBRARY_PATH
 
+    if [ -n "$DYLD_LIBRARY_PATH" ]; then
+        DYLD_LIBRARY_PATH="$libdir:$DYLD_LIBRARY_PATH"
+    else
+        DYLD_LIBRARY_PATH=$libdir
+    fi
+    export DYLD_LIBRARY_PATH
+
     # ----------
     # Windows needs shared libraries in PATH. (Only those linked into
     # executables, not dlopen'ed ones)
     # ----------
     case $host_platform in
-        *-*-cygwin*)
+        *-*-cygwin*|*-*-mingw32*)
             PATH=$libdir:$PATH
             export PATH
             ;;
@@ -389,7 +427,11 @@ then
 
     message "starting postmaster"
     [ "$debug" = yes ] && postmaster_options="$postmaster_options -d 5"
-    [ "$unix_sockets" = no ] && postmaster_options="$postmaster_options -i"
+    if [ "$unix_sockets" = no ]; then
+        postmaster_options="$postmaster_options -c listen_addresses=$hostname"
+    else
+        postmaster_options="$postmaster_options -c listen_addresses=''"
+    fi
     "$bindir/postmaster" -D "$PGDATA" -F $postmaster_options >"$LOGDIR/postmaster.log" 2>&1 &
     postmaster_pid=$!
 
@@ -398,7 +440,7 @@ then
     # wait forever, however.
     i=0
     max=60
-    until "$bindir/psql" $psql_options template1 </dev/null 2>/dev/null
+    until "$bindir/psql" -X $psql_options template1 </dev/null 2>/dev/null
     do
         i=`expr $i + 1`
         if [ $i -ge $max ]
@@ -427,12 +469,16 @@ then
 
 else # not temp-install
 
-    # If Unix sockets are not available, use the local host by default.
-    if [ "$unix_sockets" = no ]; then
-        PGHOST=$hostname
-        export PGHOST
-        unset PGHOSTADDR
-    fi
+    # ----------
+    # Windows needs shared libraries in PATH. (Only those linked into
+    # executables, not dlopen'ed ones)
+    # ----------
+    case $host_platform in
+        *-*-cygwin*|*-*-mingw32*)
+            PATH=$libdir:$PATH
+            export PATH
+            ;;
+    esac
 
     if [ -n "$PGPORT" ]; then
         port_info="port $PGPORT"
@@ -443,8 +489,13 @@ else # not temp-install
     if [ -n "$PGHOST" ]; then
         echo "(using postmaster on $PGHOST, $port_info)"
     else
-        echo "(using postmaster on Unix socket, $port_info)"
+        if [ "$unix_sockets" = no ]; then
+            echo "(using postmaster on localhost, $port_info)"
+        else
+            echo "(using postmaster on Unix socket, $port_info)"
+        fi
     fi
+
     message "dropping database \"$dbname\""
     "$bindir/dropdb" $psql_options "$dbname"
     # errors can be ignored
@@ -483,6 +534,9 @@ fi
 # Create the regression database
 # We use template0 so that any installation-local cruft in template1
 # will not mess up the tests.
+# Note: the reason for checkpointing just after creating the new DB is so
+# that if we get a backend core dump during the tests, WAL replay won't
+# remove the core file.
 # ----------
 
 message "creating database \"$dbname\""
@@ -492,11 +546,12 @@ if [ $? -ne 0 ]; then
     (exit 2); exit
 fi
 
-"$bindir/psql" $psql_options -c "\
+"$bindir/psql" -q -X $psql_options -c "\
+checkpoint;
 alter database \"$dbname\" set lc_messages to 'C';
 alter database \"$dbname\" set lc_monetary to 'C';
 alter database \"$dbname\" set lc_numeric to 'C';
-alter database \"$dbname\" set lc_time to 'C';" "$dbname" 2>/dev/null
+alter database \"$dbname\" set lc_time to 'C';" "$dbname"
 if [ $? -ne 0 ]; then
     echo "$me: could not set database default locales"
     (exit 2); exit
@@ -508,7 +563,7 @@ fi
 # ----------
 
 message "dropping regression test user accounts"
-"$bindir/psql" $psql_options -c 'DROP GROUP regressgroup1; DROP GROUP regressgroup2; DROP USER regressuser1, regressuser2, regressuser3, regressuser4;' $dbname 2>/dev/null
+"$bindir/psql" -q -X $psql_options -c 'DROP GROUP regressgroup1; DROP GROUP regressgroup2; DROP USER regressuser1, regressuser2, regressuser3, regressuser4;' $dbname 2>/dev/null
 if [ $? -eq 2 ]; then
     echo "$me: could not drop user accounts"
     (exit 2); exit
@@ -579,7 +634,8 @@ do
         # Run a single test
         formatted=`echo $1 | awk '{printf "%-20.20s", $1;}'`
         $ECHO_N "test $formatted ... $ECHO_C"
-        $PSQL -d "$dbname" <"$inputdir/sql/$1.sql" >"$outputdir/results/$1.out" 2>&1
+        ( $PSQL -d "$dbname" <"$inputdir/sql/$1.sql" >"$outputdir/results/$1.out" 2>&1 )&
+        wait
     else
         # Start a parallel group
         $ECHO_N "parallel group ($# tests): $ECHO_C"
@@ -666,7 +722,7 @@ do
                 fi
                 ;;
             2)
-                # desaster struck
+                # disaster struck
                 echo "trouble" 1>&2
                 (exit 2); exit;;
         esac
@@ -681,7 +737,7 @@ done | tee "$result_summary_file" 2>&1
 
 if [ -n "$postmaster_pid" ]; then
     message "shutting down postmaster"
-    kill -15 "$postmaster_pid"
+    "$bindir/pg_ctl" -s -D "$PGDATA" stop
     wait "$postmaster_pid"
     unset postmaster_pid
 fi
